package be.iminds.ilabt.jfed.experimenter_gui.slice.espec.view;

import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecLogItem;
import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecLogModel;
import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecStateItem;
import be.iminds.ilabt.jfed.experimenter_gui.ui.status.TaskStatusIndicator;
import javafx.beans.binding.ObjectBinding;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.StringProperty;
import javafx.collections.ListChangeListener;
import javafx.scene.Cursor;
import javafx.scene.control.*;
import javafx.scene.layout.HBox;

import javax.annotation.Nonnull;


public class ESpecLogList extends ListView<ESpecLogItem> {


    public ESpecLogList(ESpecLogModel eSpecLogModel) {
        setPrefHeight(100);
        setCellFactory(progressItemListView -> new ESpecViewItem());

        setItems(eSpecLogModel.getESpecLogItems());

        //always show the last progressitem that was added
        ListChangeListener<ESpecLogItem> showLastItem = c -> {
            while (c.next()) {
                if (c.wasAdded()) {
                    scrollTo(getItems().size() - 1);
                }
            }
        };
        getItems().addListener(showLastItem);

        eSpecLogModel.selectedESpecStateItemProperty().addListener((o, oldVal, newVal) -> {
            if (oldVal != newVal) {
                getItems().removeListener(showLastItem);
                if (newVal != null) {
                    setItems(newVal.getLogItems());
                } else {
                    setItems(eSpecLogModel.getESpecLogItems());
                }
                getItems().addListener(showLastItem);
                scrollTo(getItems().size() - 1);
            }
        });
    }


    private static class ESpecViewItem extends ListCell<ESpecLogItem> {

        private final HBox hbox;

        private final TaskStatusIndicator taskStatusIndicator;
        private final Label textLabel;
        private final Label statusLabel;

        private final Label logDetailsLabel;
        private final ProgressBar uploadProgressBar;

        public ESpecViewItem() {
            hbox = new HBox();
            hbox.setSpacing(5);


            // add taskStatusIndicator as graphic
            taskStatusIndicator = new TaskStatusIndicator();
            taskStatusIndicator.setPrefHeight(16);
            taskStatusIndicator.setPrefWidth(16);

            textLabel = new Label();
            statusLabel = new Label();
            statusLabel.setStyle("-fx-text-fill: darkgrey;");

            logDetailsLabel = new Label("");
            logDetailsLabel.setStyle("-fx-text-fill: darkgrey; -fx-font-scale: 0.75;");

            uploadProgressBar = new ProgressBar();

            hbox.getChildren().addAll(taskStatusIndicator, textLabel, statusLabel, logDetailsLabel, uploadProgressBar);

            logDetailsLabel.setVisible(false);
            uploadProgressBar.setVisible(false);
            logDetailsLabel.managedProperty().bind(logDetailsLabel.visibleProperty());
            uploadProgressBar.managedProperty().bind(uploadProgressBar.visibleProperty());

            setGraphic(hbox);
        }

        @Override
        protected void updateItem(final ESpecLogItem item, boolean empty) {
            super.updateItem(item, empty);

            logDetailsLabel.visibleProperty().unbind();
            uploadProgressBar.visibleProperty().unbind();

            if (!empty) {
                textLabel.tooltipProperty().bind(new TooltipBinding(item.eSpecStateItemProperty()));
                textLabel.textProperty().bind(item.textProperty());
                statusLabel.textProperty().bind(item.statusTextProperty());
                taskStatusIndicator.statusProperty().bind(item.statusProperty());
                taskStatusIndicator.setVisible(true);

                logDetailsLabel.visibleProperty().bind(item.logByteSizeProperty().greaterThan(0));
                logDetailsLabel.textProperty().bind(item.logByteSizeProperty().asString("(log size = %d bytes)"));

                uploadProgressBar.progressProperty().bind(item.progressProperty());
                uploadProgressBar.visibleProperty().bind(item.progressProperty().greaterThan(0.0));

                setOnMouseEntered(mouseEvent -> {
                    if (item.getOnDetailsView() != null) {
                        setOnMouseClicked(item.getOnDetailsView());
                        setCursor(Cursor.HAND);
                    } else {
                        setOnMouseClicked(null);
                        setCursor(null);
                    }
                });

            } else {
                textLabel.textProperty().unbind();
                textLabel.setText("");
                statusLabel.textProperty().unbind();
                statusLabel.setText("");
                taskStatusIndicator.setVisible(false);
                setOnMouseClicked(null);
                setCursor(null);
                logDetailsLabel.setVisible(false);
                uploadProgressBar.setVisible(false);
            }
        }
    }





    private static class TooltipStringBinding extends ObjectBinding<Tooltip> {
        @Nonnull private final StringProperty text;

        public TooltipStringBinding(StringProperty text) {
            bind(text);
            this.text = text;
        }

        @Override
        protected Tooltip computeValue() {
            return new Tooltip(text.get());
        }
    }
    private static class TooltipBinding extends ObjectBinding<Tooltip> {
        @Nonnull private final ObjectProperty<ESpecStateItem> item;

        public TooltipBinding(ObjectProperty<ESpecStateItem> item) {
            bind(item);
            this.item = item;
        }

        @Override
        protected Tooltip computeValue() {
            if (item.get() != null)
                return new Tooltip("Occured during state "+item.get().getHumanName());
            else
                return new Tooltip("Occured during null state ");
        }
    }
}
